﻿//     ========================== avm_Invoke.cs =============================
//     **********************************************************************
//     Author: Yeremeyev Dmitriy Vladimirovich *** mailto:edvmail@mail.ru
//     Last update : 30.07.2010
//     **********************************************************************

using System;
using System.Runtime.InteropServices;
using System.Drawing;
using AVM.Struct;

namespace AVM.api
{
    public partial class avmInvoke
    {
        /// <summary>
        /// The file name of the AVM library
        /// </summary>
        private const string AVM_LIBRARY = "avm061.dll";

        /// <summary>
        /// Open AVM API
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern avmHandle avmOpen(short aDataSize);

        /// <summary>
        /// Close AVM API
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmClose(avmHandle aHandle);

        /// <summary>
        /// Creating of associative memory
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmCreate(avmHandle aHandle, Size aKeyImgSize,
                                            short aLevelMax, int aTreeTotal, bool aClustering);
        /// <summary>
        /// Destroying of associative memory
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmDestroy(avmHandle aHandle);

        /// <summary>
        /// Set the active associative tree
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmSetActiveTree(avmHandle aHandle, int aTreeIdx);

        /// <summary>
        /// Removing all data out from associative tree
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmClearTreeData(avmHandle aHandle);

        /// <summary>
        /// Saving of recognition data
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern bool avmSave(avmHandle aHandle, char[] aFileName, bool aSaveWithoutOptimization);

        /// <summary>
        /// Loading of recognition data
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern bool avmLoad(avmHandle aHandle, char[] aFileName);

        /// <summary>
        /// Get size of packed recognition data
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern int avmGetPackedDataSize(avmHandle aHandle);

        /// <summary>
        /// Writing a packed recognition data to memory
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmWritePackedData(avmHandle aHandle, IntPtr apMem);

        /// <summary>
        /// Reading a packed recognition data from memory
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmReadPackedData(avmHandle aHandle, IntPtr apMem);

        /// <summary>
        /// Optimization of associative tree
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmOptimizeAssociativeTree(avmHandle aHandle);

        /// <summary>
        /// Restart time counter of optimization event
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmRestartTimeForOptimization(avmHandle aHandle);

        /// <summary>
        /// The estimation of an opportunity for training 
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern double avmEstimateOpportunityForTraining(avmHandle aHandle, Rectangle aInterestArea);

        /// <summary>
        /// Set an image for processing where image is presented as 2D pixel (byte) array
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmSetImageAsArray(avmHandle aHandle, Size aSize, IntPtr apSrcImg, IntPtr apObjMaskImg);

        /// <summary>
        /// Object recognition (result will be returned as sequence) 
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern IntPtr avmObjectRecognition_sq_32S(avmHandle aHandle, ref int apObjDsrTotal);

        /// <summary>
        /// Object recognition and tracking (result will be returned as sequence)
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern IntPtr avmObjectTracking_sq_32S(avmHandle aHandle, ref int apObjDsrTotal,
                              bool aTrainingDuringTracking, double aSimilarityUpTrd, double aSimilarityDnTrd);


        /// <summary>
        /// Reading from associative memory cell (associative base)
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern bool avmRead_32S(avmHandle aHandle, Rectangle aInterestArea, ref Rectangle apSearchArea,
                                              ref IntPtr appData, ref UInt64 apIndex, ref UInt64 apHitCounter,
                                              ref double apSimilarity, bool aTotalSearch);

        /// <summary>
        /// Writing to associative memory cell
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmWrite_32S(avmHandle aHandle, Rectangle aInterestArea,
                                                         ref int apData, bool aDeepLearning);
        /// <summary>
        /// Get total number of associative bases
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern int avmGetTotalABases(avmHandle aHandle);

        /// <summary>
        /// Get total number of memory levels
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern short avmGetTotalLevels(avmHandle aHandle);

        /// <summary>
        /// Get current index of associative base
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern UInt64 avmGetCurIndex(avmHandle aHandle);

        /// <summary>
        /// Get read-write counter
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern UInt64 avmGetWrRdCounter(avmHandle aHandle);

        /// <summary>
        /// Get size of key image of associative memory
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern Size avmGetKeyImageSize(avmHandle aHandle);

        /// <summary>
        /// Get base key size
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern Size avmGetBaseKeySize(avmHandle aHandle);

        /// <summary>
        /// Set the value of parameter
        /// </summary>
        [DllImport(AVM_LIBRARY)]
        public static extern void avmSetParam(avmHandle aHandle, CvAM_ParamType aParam, double aValue);
    }
}
