﻿//     ========================== avm_Struct.cs =============================
//     **********************************************************************
//     Author: Yeremeyev Dmitriy Vladimirovich *** mailto:edvmail@mail.ru
//     Last update : 30.07.2010
//     **********************************************************************

using System;
using System.Runtime.InteropServices;
using System.Drawing;

namespace AVM.Struct
{
    /// <summary>
    /// Wrapper for handle of AVM API
    /// </summary>
    public struct avmHandle
    {
        public ushort h;
    }

    /// <summary>
    /// Definition of parameter types
    /// </summary>
    public enum CvAM_ParamType {
	        ptOptimizeLoop = 0,      // Period of optimization, ms (45000)
		    ptTrackingDepth,         // Depth of tracking (40)
		    ptTrcDist2Max,           // Threshold of square distance for neighbor objects
		    ptExptectDistEr,         // Expected distance error for tracking
		    ptRcgMxCellBase,         // Base cell value for recognition threshold calculation (20000)
		    ptMaxClusterAge,         // Age of formed cluster (40)
		    ptReserved1,             // Not used parameter
		    ptReserved2,             // Not used parameter
		    ptReserved3,             // Not used parameter
		    ptSearchScaleStep        // Scale step of searching  for recognition (0.25)
    };

    /// <summary>
    /// Definition of the state constants for recognition function
    /// </summary>
    public enum CvAM_State
    {
        cRecognizedObject = 2,
        cGeneralizedObject,
        cTrackedObject,
        cLearnThisObject
    };

    /// <summary>
    /// Data structure of tracking
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct CvTrcInfo
    {
        /// <summary>
        /// Length of tracking
        /// </summary>
        public const int cAM_TrcLen = 100;
        /// <summary>
        /// Trajectory points
        /// </summary>
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = cAM_TrcLen)]
        public Point[] Pnt;
        /// <summary>
        /// Age of tracking
        /// </summary>
        public int Age;
    }

    /// <summary>
    /// Definition of object descriptor for type "long"
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct CvObjDsr32S
    {
        /// <summary>
        /// State of recognition function
        /// </summary>
        public CvAM_State State;
        /// <summary>
        /// Rectangle where object found
        /// </summary>
        public Rectangle ObjRect;
        /// <summary>
        // Similarity of the interest area to an object (0 ... 1)
        /// </summary>
        public float Similarity;
        /// <summary>
        /// Associated data
        /// </summary>
        public int Data;
        /// <summary>
        /// Trajectory
        /// </summary>
        public CvTrcInfo Trj;
    }

    /// <summary>
    /// Recognition data structure
    /// </summary>
    [Serializable]
    public struct CvRcgData
    {
        public int Size;
        public byte[] Data;
    }
}
