﻿#region Using directives

using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using AviFile;
using AVM.api;
using AVM.Struct;

#endregion

namespace RcgExample1 {
    partial class MainForm : Form {
        private const string cVideoFileName = "RcgTraining4s.avi";
        private const string cRcgDataFileName = "am_example.dat";
        private RcgPipeline mPlayer;
        private EditableVideoStream mTestVideo;
        private CvAssociativeMemory32S _am;

        [STAThread]
        public static void Main() {
            Application.Run(new MainForm());
        }
        
        public MainForm() {
            InitializeComponent();

            _am = new CvAssociativeMemory32S();

            if (_am.Load(cRcgDataFileName))
            {
                // Test of removing all data out from associative tree
                //_am.ClearTreeData();

                // Increase the base cell value for recognition threshold calculation
                //_am.SetParam(CvAM_ParamType.ptRcgMxCellBase, 25000);

                // Set the period of optimization every 4 seconds
                //_am.SetParam(CvAM_ParamType.ptOptimizeLoop, 4000);

                // Set the age of formed cluster to 70 hits
                //_am.SetParam(CvAM_ParamType.ptMaxClusterAge, 70);

                // Set the depth of tracking to 4 frames
                //_am.SetParam(CvAM_ParamType.ptTrackingDepth, 4);

                LogBox.Items.Add(String.Format("Load associative bases from file: {0:S}", cRcgDataFileName));
                LogBox.Items.Add(String.Format("   Base key size ............. {0:D}x{1:D}",
                                                    _am.GetBaseKeySize().Width, _am.GetBaseKeySize().Height));
                LogBox.Items.Add(String.Format("   Associative bases ......... {0:D}", _am.GetTotalABases()));
                LogBox.Items.Add(String.Format("   Associative Levels ........ {0:D}", _am.GetTotalLevels()));
                LogBox.Items.Add(String.Format("   Current base index ........ {0:D}", _am.GetCurIndex()));
                LogBox.Items.Add(String.Format("   Wr/Rd counter ............. {0:D}", _am.GetWrRdCounter()));
            }
            else
            {
                LogBox.Items.Add(String.Format("Recognition data is not loaded. File: \"{0:S}\" not found.", cRcgDataFileName));
                Size KeyImgSize = new Size();
                KeyImgSize.Width = 80;
                KeyImgSize.Height = 80;
                _am.Create(KeyImgSize, 0, 1, true);
            }

            AviManager file = new AviManager(cVideoFileName, true);
            VideoStream stream = file.GetVideoStream();
            mTestVideo = new EditableVideoStream(stream);
            file.Close();
            mPlayer = new RcgPipeline(mTestVideo, picPreview, ref _am);
            mPlayer.Start();
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            mPlayer.Stop();
            base.OnFormClosing(e);
        }
    }

    partial class RcgPipeline : AviPlayer
    {
        private const int cObjMaxNum = 4; // Maximal number of recognition objects
        private int _PauseLoop = 200;
        private int _TrainingLoop = 600;
        private CvAssociativeMemory32S _am;

        public RcgPipeline(VideoStream aVideoStream, PictureBox aPicDisplay, ref CvAssociativeMemory32S am) :
            base(aVideoStream, aPicDisplay, null) { _am = am; }

        /// <summary>Frame capture function</summary>
        protected override void GrabCurrentFrame(ref Bitmap aPicDisplay)
        {
            _am.SetImage(ref aPicDisplay);

            Graphics DisplayGraphics = Graphics.FromImage(aPicDisplay);

            Rectangle InterestArea = new Rectangle();
            InterestArea.X = 84;
            InterestArea.Y = 38;
            InterestArea.Width = 80;
            InterestArea.Height = 80;

            // Create font and brush
            Font drawFont = new Font("Arial", 16);
            Font drawFontSm = new Font("Arial", 8);
            // Set format of string
            StringFormat drawFormat = new StringFormat();

            if (_PauseLoop != 0)
            {
                DisplayGraphics.DrawRectangle(new Pen(Color.Green, 2), InterestArea);
                // Draw estimation of an opportunity for training
                DisplayGraphics.DrawString(String.Format("EOT: {0:F}",
                    _am.EstimateOpportunityForTraining(InterestArea)),
                    drawFont, new SolidBrush(Color.GreenYellow),
                    new Point(InterestArea.X, InterestArea.Y + InterestArea.Height), drawFormat);
                _PauseLoop--;
            }
            else
            {
                if (_TrainingLoop != 0)
                {
                    // Training
                    int wObjIndex = cObjMaxNum;// <= Actual index of training object (Now it lastest object)
                    _am.Write(InterestArea, wObjIndex, true);

                    DisplayGraphics.DrawRectangle(new Pen(Color.Red, 2), InterestArea);
                    _TrainingLoop--;
                }
                else
                {
                    // Object recognition
                    Array SeqObjDsr = _am.ObjectTracking(true, 0.54, 0.45);

                    for (int i = 0; SeqObjDsr != null && i < SeqObjDsr.Length; i++)
                    {
                        CvObjDsr32S ObjDsr = (CvObjDsr32S)SeqObjDsr.GetValue(i);

                        switch(ObjDsr.State) {
                            case CvAM_State.cRecognizedObject:
                                DisplayGraphics.DrawRectangle(new Pen(Color.LightSkyBlue, 1), ObjDsr.ObjRect);
                                break;

                            case CvAM_State.cGeneralizedObject:
                            case CvAM_State.cTrackedObject:
                                DisplayGraphics.DrawRectangle(new Pen(Color.Blue, 2), ObjDsr.ObjRect);

                                // Draw object trajectory
                                if (ObjDsr.Trj.Age > 2)
                                {
                                    for (int t = 1; t < ObjDsr.Trj.Age; t++)
                                    {
                                        DisplayGraphics.DrawLine(new Pen(Color.AliceBlue, 2),
                                            ObjDsr.Trj.Pnt[CvTrcInfo.cAM_TrcLen - t - 1],
                                            ObjDsr.Trj.Pnt[CvTrcInfo.cAM_TrcLen - t]);
                                    }
                                    Point ObjCenter = ObjDsr.Trj.Pnt[CvTrcInfo.cAM_TrcLen - 1];
                                    DisplayGraphics.DrawEllipse(new Pen(Color.LimeGreen, 2), ObjCenter.X, ObjCenter.Y, 4, 4);
                                }

                                string[] ObjName = { "Mr. Dmitriy", "Toy \"Heart\"",
                                             "Book", "Post card"}; //"your name of object", ...  };

                                // Draw object name to screen
                                DisplayGraphics.DrawString(ObjName[ObjDsr.Data - 1],
                                    drawFont, new SolidBrush(Color.Yellow),
                                    new Point(ObjDsr.ObjRect.X, ObjDsr.ObjRect.Y +
                                              ObjDsr.ObjRect.Height), drawFormat);

                                // Draw similarity
                                DisplayGraphics.FillRectangle(new SolidBrush(Color.Black),
                                    new Rectangle(ObjDsr.ObjRect.X, ObjDsr.ObjRect.Y +
                                                  ObjDsr.ObjRect.Height - 10, 24, 14));

                                DisplayGraphics.DrawString(String.Format("{0:F}", ObjDsr.Similarity),
                                    drawFontSm, new SolidBrush(Color.GreenYellow),
                                    new Point(ObjDsr.ObjRect.X, ObjDsr.ObjRect.Y +
                                              ObjDsr.ObjRect.Height-10), drawFormat);
                                break;

                            case CvAM_State.cLearnThisObject:
                                DisplayGraphics.DrawRectangle(new Pen(Color.LightCyan, 4), ObjDsr.ObjRect);
                                break;

                        }
                    }
                }
                // Draw total number of associative bases
                DisplayGraphics.DrawString(String.Format("Associative bases: {0:D}", _am.GetTotalABases()),
                    drawFont, new SolidBrush(Color.GreenYellow),
                    new Point(0, 0), drawFormat);
            }
        }
    }
}